@@ -1,4 +1,4 @@
-from peak.rules import abstract, when
+from simplegeneric import generic
 import schemaish
 
 try:
@@ -179,12 +179,9 @@ class TimeToStringConverter(Converter):
             raise ConvertError('Invalid time: '+str(e))
             
         return value
-        
  
 
     
-    
-    
 class DateToDateTupleConverter(Converter):
     
     def from_type(self, value, converter_options={}):
@@ -208,7 +205,6 @@ class DateToDateTupleConverter(Converter):
 
 
 
-
 def getDialect(delimiter=','):
     import csv
     class Dialect(csv.excel):
@@ -342,89 +338,210 @@ class TupleToStringConverter(Converter):
         delimiter = converter_options.get('delimiter',',')
         l = convert_csvrow_to_list(value, delimiter=delimiter)
         convl = [string_converter(self.schema_type.attrs[n]).to_type(v) \
-                 for n,v in enumerate(l)]
+                 for n,v in enumerate(l[:len(self.schema_type.attrs)])]
         return tuple(convl)
+
+
+class TupleToListConverter(Converter):
+
+    def __init__(self, schem_type, **k):
+        Converter.__init__(self, schema_type, **k)
+
+    def from_type(self, value, converter_options={}):
+        if value is None:
+            return None
+        return list(value)
+
+    def to_type(self, value, converter_options={}):
+        if value is None:
+            return None
+        return tuple(value)
+
+
+
+####
+#
+#  String Converter
+
     
     
-@abstract()
+@generic
 def string_converter(schema_type):
     pass
 
 
-@when(string_converter, (schemaish.String,))
+@string_converter.when_type(schemaish.String)
 def string_to_string(schema_type):
     return NullConverter(schema_type)
 
-@when(string_converter, (schemaish.Integer,))
+@string_converter.when_type(schemaish.Integer)
 def int_to_string(schema_type):
     return IntegerToStringConverter(schema_type)
 
-@when(string_converter, (schemaish.Float,))
+@string_converter.when_type(schemaish.Float)
 def float_to_string(schema_type):
     return FloatToStringConverter(schema_type)
 
-@when(string_converter, (schemaish.Decimal,))
+@string_converter.when_type(schemaish.Decimal)
 def decimal_to_string(schema_type):
     return DecimalToStringConverter(schema_type)
 
-@when(string_converter, (schemaish.Date,))
+@string_converter.when_type(schemaish.Date)
 def date_to_string(schema_type):
     return DateToStringConverter(schema_type)
 
-@when(string_converter, (schemaish.Time,))
+@string_converter.when_type(schemaish.Time)
 def time_to_string(schema_type):
     return TimeToStringConverter(schema_type)
 
-@when(string_converter, (schemaish.Sequence,))
+@string_converter.when_type(schemaish.Sequence)
 def sequence_to_string(schema_type):
     return SequenceToStringConverter(schema_type)
 
-@when(string_converter, (schemaish.Tuple,))
+@string_converter.when_type(schemaish.Tuple)
 def tuple_to_string(schema_type):
     return TupleToStringConverter(schema_type)
 
-@when(string_converter, (schemaish.Boolean,))
+@string_converter.when_type(schemaish.Boolean)
 def boolean_to_string(schema_type):
     return BooleanToStringConverter(schema_type)
 
-@when(string_converter, (schemaish.File,))
+@string_converter.when_type(schemaish.File)
 def file_to_string(schema_type):
     return FileToStringConverter(schema_type)
 
+####
+#
+#  Date Tuple Converter
 
-@abstract()
+
+@generic
 def datetuple_converter(schema_type):
     pass
 
-@when(datetuple_converter, (schemaish.Date,))
+@datetuple_converter.when_type(schemaish.Date)
 def date_to_datetuple(schema_type):
     return DateToDateTupleConverter(schema_type)
 
+####
+#
+#  Boolean Converter
 
 
-@abstract()
+@generic
 def boolean_converter(schema_type):
     pass
 
-@when(boolean_converter, (schemaish.Boolean,))
+@boolean_converter.when_type(schemaish.Boolean)
 def boolean_to_boolean(schema_type):
     return NullConverter(schema_type)
 
 
 
-@abstract()
+@generic
 def file_converter(schema_type):
     pass
 
-@when(file_converter, (schemaish.File,))
+@file_converter.when_type(schemaish.File)
 def file_to_file(schema_type):
     return NullConverter(schema_type)
 
 
+####
+#
+#  JSON Converter
+
+
+class DateToJSONConverter(Converter):
+    
+    def from_type(self, value, converter_options={}):
+        if value is None:
+            return None
+        return {'__type__':'date','year':value.year, 'month':value.month, 'day':value.day}
+        
+    def to_type(self, value, converter_options={}):
+        if value is None:
+            return None
+        try:
+            try:
+                year, month, day = int(value['year']) ,int(value['month']) ,int(value['day']) ,
+            except ValueError:
+                raise ConvertError('Invalid Number')
+            value = date(year, month, day)
+        except (TypeError, ValueError), e:
+            raise ConvertError('Invalid date: '+str(e))
+        return value
+
+class TimeToJSONConverter(Converter):
+    
+    def from_type(self, value, converter_options={}):
+        if value is None:
+            return None
+        return {'__type__':'time','hour':value.hour, 'minute':value.minute, 'second':value.second,'microsecond':value.microsecond}
+        
+    def to_type(self, value, converter_options={}):
+        if value is None:
+            return None
+        try:
+            try:
+                h, m, s, ms = int(value['hour']), int(value['minute']), int(value['second']) ,int(value.get('microsecond',0))
+            except ValueError:
+                raise ConvertError('Invalid Number')
+            value = time(h, m, s, ms)
+        except (TypeError, ValueError), e:
+            raise ConvertError('Invalid time: '+str(e))
+        return value
+
+
+@generic
+def json_converter(schema_type):
+    pass
+
+@json_converter.when_type(schemaish.String)
+def string_to_json(schema_type):
+    return NullConverter(schema_type)
+
+@json_converter.when_type(schemaish.Integer)
+def int_to_json(schema_type):
+    return NullConverter(schema_type)
+
+@json_converter.when_type(schemaish.Float)
+def float_to_json(schema_type):
+    return NullConverter(schema_type)
+
+# XXX
+@json_converter.when_type(schemaish.Decimal)
+def decimal_to_json(schema_type):
+    return NullConverter(schema_type)
+
+@json_converter.when_type(schemaish.Date)
+def date_to_json(schema_type):
+    return DateToJSONConverter(schema_type)
+
+@json_converter.when_type(schemaish.Time)
+def time_to_json(schema_type):
+    return TimeToJSONConverter(schema_type)
+
+@json_converter.when_type(schemaish.Sequence)
+def sequence_to_json(schema_type):
+    return NullConverter(schema_type)
+
+@json_converter.when_type(schemaish.Tuple)
+def tuple_to_json(schema_type):
+    return TupleToListConverter(schema_type)
+
+@json_converter.when_type(schemaish.Boolean)
+def boolean_to_json(schema_type):
+    return NullConverter(schema_type)
+
+@json_converter.when_type(schemaish.File)
+def file_to_json(schema_type):
+    return FileToStringConverter(schema_type)
+
 
 
 
 __all__ = [
     'string_converter', 'datetuple_converter',
-    'boolean_converter', 'file_converter'
+    'boolean_converter', 'file_converter','json_converter'
     ]
